@@ -61,7 +61,8 @@ Convolver::Convolver(const std::string& tag, pw::Manager* pipe_manager, Pipeline
       settings(
           db::Manager::self().get_plugin_db<db::Convolver>(pipe_type,
                                                            tags::plugin_name::BaseName::convolver + "#" + instance_id)),
-      kernel_manager(ConvolverKernelManager(pipe_type)) {
+      kernel_manager(ConvolverKernelManager(pipe_type)),
+      worker(new ConvolverWorker) {
   init_common_controls<db::Convolver>(settings);
 
   dry = (settings->dry() <= util::minimum_db_d_level) ? 0.0F : static_cast<float>(util::db_to_linear(settings->dry()));
@@ -92,15 +93,13 @@ Convolver::Convolver(const std::string& tag, pw::Manager* pipe_manager, Pipeline
 
   // Preparing the worker thread
 
-  worker = new ConvolverWorker;
-
   worker->moveToThread(&workerThread);
 
   connect(&workerThread, &QThread::finished, worker, &QObject::deleteLater);
 
   connect(
       worker, &ConvolverWorker::onNewKernel, this,
-      [this](ConvolverKernelManager::KernelData data) {
+      [this](ConvolverKernelManager::KernelData data, bool init_zita) {
         kernel_is_initialized = data.isValid();
 
         if (kernel_is_initialized) {
@@ -113,15 +112,17 @@ Convolver::Convolver(const std::string& tag, pw::Manager* pipe_manager, Pipeline
           Q_EMIT kernelSamplesChanged();
           Q_EMIT newKernelLoaded(data.name, true);
 
-          std::scoped_lock<std::mutex> lock(data_mutex);
+          if (init_zita) {
+            std::scoped_lock<std::mutex> lock(data_mutex);
 
-          auto success = zita.init(data, blocksize, settings->irWidth(), settings->autogain());
+            auto success = zita.init(data, blocksize, settings->irWidth(), settings->autogain());
 
-          if (!success) {
-            util::warning(std::format("{} Zita init failed", log_tag));
+            if (!success) {
+              util::warning(std::format("{} Zita init failed", log_tag));
+            }
+
+            ready = success;
           }
-
-          ready = success;
         }
       },
       Qt::QueuedConnection);
@@ -164,8 +165,6 @@ Convolver::Convolver(const std::string& tag, pw::Manager* pipe_manager, Pipeline
       },
       Qt::QueuedConnection);
 
-  workerThread.start();
-
   QMetaObject::invokeMethod(
       worker,
       [this] {
@@ -199,9 +198,6 @@ Convolver::~Convolver() {
 
   zita.stop();
 
-  workerThread.quit();
-  workerThread.wait();
-
   disconnect();
   settings->disconnect();
 
@@ -357,7 +353,7 @@ void Convolver::process([[maybe_unused]] std::span<float>& left_in,
                         [[maybe_unused]] std::span<float>& probe_right) {}
 
 void Convolver::load_kernel_file(const bool& init_zita, const uint& server_sampling_rate) {
-  if (destructor_called || server_sampling_rate == 0) {
+  if (destructor_called) {
     return;
   }
 
@@ -371,14 +367,14 @@ void Convolver::load_kernel_file(const bool& init_zita, const uint& server_sampl
     return;
   }
 
-  if (kernel_data.rate != server_sampling_rate) {
+  if (server_sampling_rate != 0 && kernel_data.rate != server_sampling_rate) {
     util::debug(std::format("{}{} kernel has {} rate. Resampling it to {}", log_tag, name.toStdString(),
                             kernel_data.rate, server_sampling_rate));
 
     kernel_data = ConvolverKernelManager::resampleKernel(kernel_data, server_sampling_rate);
   }
 
-  const auto dt = 1.0 / server_sampling_rate;
+  const auto dt = 1.0 / kernel_data.rate;
 
   std::vector<double> time_axis(kernel_data.sampleCount());
 
@@ -390,11 +386,11 @@ void Convolver::load_kernel_file(const bool& init_zita, const uint& server_sampl
 
   std::vector<double> copy_helper(kernel_data.sampleCount());
 
-  std::ranges::copy(kernel_data.left_channel, copy_helper.begin());
+  std::ranges::copy(kernel_data.channel_L, copy_helper.begin());
 
   auto magL = util::interpolate(time_axis, copy_helper, x_linear);
 
-  std::ranges::copy(kernel_data.right_channel, copy_helper.begin());
+  std::ranges::copy(kernel_data.channel_R, copy_helper.begin());
 
   auto magR = util::interpolate(time_axis, copy_helper, x_linear);
 
@@ -410,15 +406,13 @@ void Convolver::load_kernel_file(const bool& init_zita, const uint& server_sampl
 
   ConvolverKernelFFT kernel_fft;
 
-  kernel_fft.calculate_fft(kernel_data.left_channel, kernel_data.right_channel, kernel_data.rate, interpPoints);
+  kernel_fft.calculate_fft(kernel_data.channel_L, kernel_data.channel_R, kernel_data.rate, interpPoints);
 
   Q_EMIT worker->onNewChartMag(chart_mag_L, chart_mag_R);
 
   Q_EMIT worker->onNewSpectrum(kernel_fft.linear_L, kernel_fft.linear_R, kernel_fft.log_L, kernel_fft.log_R);
 
-  if (init_zita) {
-    Q_EMIT worker->onNewKernel(kernel_data);
-  }
+  Q_EMIT worker->onNewKernel(kernel_data, init_zita);
 }
 
 auto Convolver::get_latency_seconds() -> float {
@@ -442,12 +436,6 @@ void Convolver::combineKernels(const QString& kernel1, const QString& kernel2, c
       Qt::QueuedConnection);
 }
 
-void Convolver::chart_kernel_fft(const std::vector<float>& kernel_L,
-                                 const std::vector<float>& kernel_R,
-                                 const float& kernel_rate) {
-  kernel_fft.calculate_fft(kernel_L, kernel_R, kernel_rate, interpPoints);
-}
-
 void Convolver::clear_chart_data() {
   chartMagL.clear();
   chartMagR.clear();
@@ -456,19 +444,3 @@ void Convolver::clear_chart_data() {
   chartMagLfftLog.clear();
   chartMagRfftLog.clear();
 }
-
-void ConvolverWorker::calculateFFT(std::vector<float> kernel_L,
-                                   std::vector<float> kernel_R,
-                                   int kernel_rate,
-                                   int interpPoints) {
-  ConvolverKernelFFT kernel_fft;
-
-  kernel_fft.calculate_fft(kernel_L, kernel_R, kernel_rate, interpPoints);
-
-  auto linear_L = kernel_fft.linear_L;
-  auto linear_R = kernel_fft.linear_R;
-  auto log_L = kernel_fft.log_L;
-  auto log_R = kernel_fft.log_R;
-
-  Q_EMIT onNewSpectrum(linear_L, linear_R, log_L, log_R);
-}
\ No newline at end of file
