@@ -20,6 +20,7 @@
 #include <kicontheme.h>
 #include <klocalizedcontext.h>
 #include <qcontainerfwd.h>
+#include <qhashfunctions.h>
 #include <qnamespace.h>
 #include <qobject.h>
 #include <qqml.h>
@@ -50,6 +51,7 @@
 #include "config.h"
 #include "db_manager.hpp"
 #include "global_shortcuts.hpp"
+#include "kcolor_manager.hpp"
 #include "local_client.hpp"
 #include "local_server.hpp"
 #include "pipeline_type.hpp"
@@ -148,15 +150,15 @@ struct UiState {
 
 static void initGlobalBypass(StreamInputEffects& sie, StreamOutputEffects& soe) {
   auto update_bypass_state = [&]() {
-    soe.set_bypass(db::Main::bypass());
-    sie.set_bypass(db::Main::bypass());
+    soe.set_bypass(DbMain::bypass());
+    sie.set_bypass(DbMain::bypass());
 
-    util::info((db::Main::bypass() ? "Enabling global bypass" : "Disabling global bypass"));
+    util::info((DbMain::bypass() ? "Enabling global bypass" : "Disabling global bypass"));
   };
 
   update_bypass_state();
 
-  QObject::connect(db::Main::self(), &db::Main::bypassChanged, update_bypass_state);
+  QObject::connect(DbMain::self(), &DbMain::bypassChanged, update_bypass_state);
 }
 
 static void initGlobalShortcuts(GlobalShortcuts* shortcuts) {
@@ -167,7 +169,7 @@ static void initGlobalShortcuts(GlobalShortcuts* shortcuts) {
     const auto desktop = qEnvironmentVariable("XDG_CURRENT_DESKTOP");
 
     if (session == "KDE" || desktop == "KDE") {
-      if (!db::Main::xdgGlobalShortcutsBound()) {
+      if (!DbMain::xdgGlobalShortcutsBound()) {
         shortcuts->bind_shortcuts();
       }
     } else {
@@ -177,16 +179,16 @@ static void initGlobalShortcuts(GlobalShortcuts* shortcuts) {
   };
 
   QObject::connect(shortcuts, &GlobalShortcuts::onBindShortcuts, [bind]() {
-    if (db::Main::xdgGlobalShortcuts()) {
+    if (DbMain::xdgGlobalShortcuts()) {
       bind();
     }
   });
 
-  QObject::connect(db::Main::self(), &db::Main::xdgGlobalShortcutsChanged, [bind]() {
-    if (db::Main::xdgGlobalShortcuts()) {
+  QObject::connect(DbMain::self(), &DbMain::xdgGlobalShortcutsChanged, [bind]() {
+    if (DbMain::xdgGlobalShortcuts()) {
       bind();
     } else {
-      db::Main::setXdgGlobalShortcutsBound(false);
+      DbMain::setXdgGlobalShortcutsBound(false);
     }
   });
 }
@@ -210,7 +212,7 @@ static void initQml(QQmlApplicationEngine& engine,
         ui.window->setPersistentGraphics(false);
         ui.window->setPersistentSceneGraph(false);
 
-        if (show_window) {
+        if (show_window && !DbMain::noWindowAfterStarting()) {
           ui.window->show();
           ui.window->raise();
           ui.window->requestActivate();
@@ -357,10 +359,11 @@ int main(int argc, char* argv[]) {
   auto local_server = std::make_unique<LocalServer>();
   auto global_shortcuts = std::make_unique<GlobalShortcuts>();
   auto autostart = std::make_unique<Autostart>();
+  auto color_manager = std::make_unique<KColorManager>();
 
   // theme initialization
 
-  if (db::Main::forceBreezeTheme()) {
+  if (DbMain::forceBreezeTheme()) {
     QApplication::setStyle(QStringLiteral("breeze"));
   }
 
